@@ -1,4 +1,4 @@
-/* $PostgreSQL: pgsql/src/interfaces/ecpg/preproc/output.c,v 1.25 2009/06/11 14:49:13 momjian Exp $ */
+/* $PostgreSQL: pgsql/src/interfaces/ecpg/preproc/output.c,v 1.26 2009/12/16 10:15:06 meskes Exp $ */
 
 #include "postgres_fe.h"
 
@@ -95,22 +95,9 @@ hashline_number(void)
 #endif
 		)
 	{
-		/* "* 2" here is for escaping '\' and '"' below */
-		char	   *line = mm_alloc(strlen("\n#line %d \"%s\"\n") + sizeof(int) * CHAR_BIT * 10 / 3 + strlen(input_filename) * 2);
-		char	   *src,
-				   *dest;
+		char	   *line = mm_alloc(strlen("\n#line %d \"%s\"\n") + sizeof(int) * CHAR_BIT * 10 / 3 + strlen(input_filename));
 
-		sprintf(line, "\n#line %d \"", yylineno);
-		src = input_filename;
-		dest = line + strlen(line);
-		while (*src)
-		{
-			if (*src == '\\' || *src == '"')
-				*dest++ = '\\';
-			*dest++ = *src++;
-		}
-		*dest = '\0';
-		strcat(dest, "\"\n");
+		sprintf(line, "\n#line %d \"%s\"\n", yylineno, input_filename);
 
 		return line;
 	}
@@ -118,14 +105,24 @@ hashline_number(void)
 	return EMPTY;
 }
 
+static char *ecpg_statement_type_name[] = {
+	"ECPGst_normal",
+	"ECPGst_execute",
+	"ECPGst_exec_immediate",
+	"ECPGst_prepnormal"
+};
+
 void
 output_statement(char *stmt, int whenever_mode, enum ECPG_statement_type st)
 {
-
 	fprintf(yyout, "{ ECPGdo(__LINE__, %d, %d, %s, %d, ", compat, force_indicator, connection ? connection : "NULL", questionmarks);
-	if (st == ECPGst_normal)
+	if (st == ECPGst_execute || st == ECPGst_exec_immediate)
 	{
-		if (auto_prepare)
+		fprintf(yyout, "%s, %s, ", ecpg_statement_type_name[st], stmt);
+	}
+	else
+	{
+		if (st == ECPGst_prepnormal && auto_prepare)
 			fputs("ECPGst_prepnormal, \"", yyout);
 		else
 			fputs("ECPGst_normal, \"", yyout);
@@ -133,8 +130,6 @@ output_statement(char *stmt, int whenever_mode, enum ECPG_statement_type st)
 		output_escaped_str(stmt, false);
 		fputs("\", ", yyout);
 	}
-	else
-		fprintf(yyout, "%d, %s, ", st, stmt);
 
 	/* dump variables to C file */
 	dump_variables(argsinsert, 1);
